# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class CatalanStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"", -1, 7),
        Among(u"\u00B7", 0, 6),
        Among(u"\u00E0", 0, 1),
        Among(u"\u00E1", 0, 1),
        Among(u"\u00E8", 0, 2),
        Among(u"\u00E9", 0, 2),
        Among(u"\u00EC", 0, 3),
        Among(u"\u00ED", 0, 3),
        Among(u"\u00EF", 0, 3),
        Among(u"\u00F2", 0, 4),
        Among(u"\u00F3", 0, 4),
        Among(u"\u00FA", 0, 5),
        Among(u"\u00FC", 0, 5)
    ]

    a_1 = [
        Among(u"la", -1, 1),
        Among(u"-la", 0, 1),
        Among(u"sela", 0, 1),
        Among(u"le", -1, 1),
        Among(u"me", -1, 1),
        Among(u"-me", 4, 1),
        Among(u"se", -1, 1),
        Among(u"-te", -1, 1),
        Among(u"hi", -1, 1),
        Among(u"'hi", 8, 1),
        Among(u"li", -1, 1),
        Among(u"-li", 10, 1),
        Among(u"'l", -1, 1),
        Among(u"'m", -1, 1),
        Among(u"-m", -1, 1),
        Among(u"'n", -1, 1),
        Among(u"-n", -1, 1),
        Among(u"ho", -1, 1),
        Among(u"'ho", 17, 1),
        Among(u"lo", -1, 1),
        Among(u"selo", 19, 1),
        Among(u"'s", -1, 1),
        Among(u"las", -1, 1),
        Among(u"selas", 22, 1),
        Among(u"les", -1, 1),
        Among(u"-les", 24, 1),
        Among(u"'ls", -1, 1),
        Among(u"-ls", -1, 1),
        Among(u"'ns", -1, 1),
        Among(u"-ns", -1, 1),
        Among(u"ens", -1, 1),
        Among(u"los", -1, 1),
        Among(u"selos", 31, 1),
        Among(u"nos", -1, 1),
        Among(u"-nos", 33, 1),
        Among(u"vos", -1, 1),
        Among(u"us", -1, 1),
        Among(u"-us", 36, 1),
        Among(u"'t", -1, 1)
    ]

    a_2 = [
        Among(u"ica", -1, 4),
        Among(u"l\u00F3gica", 0, 3),
        Among(u"enca", -1, 1),
        Among(u"ada", -1, 2),
        Among(u"ancia", -1, 1),
        Among(u"encia", -1, 1),
        Among(u"\u00E8ncia", -1, 1),
        Among(u"\u00EDcia", -1, 1),
        Among(u"logia", -1, 3),
        Among(u"inia", -1, 1),
        Among(u"\u00EDinia", 9, 1),
        Among(u"eria", -1, 1),
        Among(u"\u00E0ria", -1, 1),
        Among(u"at\u00F2ria", -1, 1),
        Among(u"alla", -1, 1),
        Among(u"ella", -1, 1),
        Among(u"\u00EDvola", -1, 1),
        Among(u"ima", -1, 1),
        Among(u"\u00EDssima", 17, 1),
        Among(u"qu\u00EDssima", 18, 5),
        Among(u"ana", -1, 1),
        Among(u"ina", -1, 1),
        Among(u"era", -1, 1),
        Among(u"sfera", 22, 1),
        Among(u"ora", -1, 1),
        Among(u"dora", 24, 1),
        Among(u"adora", 25, 1),
        Among(u"adura", -1, 1),
        Among(u"esa", -1, 1),
        Among(u"osa", -1, 1),
        Among(u"assa", -1, 1),
        Among(u"essa", -1, 1),
        Among(u"issa", -1, 1),
        Among(u"eta", -1, 1),
        Among(u"ita", -1, 1),
        Among(u"ota", -1, 1),
        Among(u"ista", -1, 1),
        Among(u"ialista", 36, 1),
        Among(u"ionista", 36, 1),
        Among(u"iva", -1, 1),
        Among(u"ativa", 39, 1),
        Among(u"n\u00E7a", -1, 1),
        Among(u"log\u00EDa", -1, 3),
        Among(u"ic", -1, 4),
        Among(u"\u00EDstic", 43, 1),
        Among(u"enc", -1, 1),
        Among(u"esc", -1, 1),
        Among(u"ud", -1, 1),
        Among(u"atge", -1, 1),
        Among(u"ble", -1, 1),
        Among(u"able", 49, 1),
        Among(u"ible", 49, 1),
        Among(u"isme", -1, 1),
        Among(u"ialisme", 52, 1),
        Among(u"ionisme", 52, 1),
        Among(u"ivisme", 52, 1),
        Among(u"aire", -1, 1),
        Among(u"icte", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ici", -1, 1),
        Among(u"\u00EDci", -1, 1),
        Among(u"logi", -1, 3),
        Among(u"ari", -1, 1),
        Among(u"tori", -1, 1),
        Among(u"al", -1, 1),
        Among(u"il", -1, 1),
        Among(u"all", -1, 1),
        Among(u"ell", -1, 1),
        Among(u"\u00EDvol", -1, 1),
        Among(u"isam", -1, 1),
        Among(u"issem", -1, 1),
        Among(u"\u00ECssem", -1, 1),
        Among(u"\u00EDssem", -1, 1),
        Among(u"\u00EDssim", -1, 1),
        Among(u"qu\u00EDssim", 73, 5),
        Among(u"amen", -1, 1),
        Among(u"\u00ECssin", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"ificar", 77, 1),
        Among(u"egar", 77, 1),
        Among(u"ejar", 77, 1),
        Among(u"itar", 77, 1),
        Among(u"itzar", 77, 1),
        Among(u"fer", -1, 1),
        Among(u"or", -1, 1),
        Among(u"dor", 84, 1),
        Among(u"dur", -1, 1),
        Among(u"doras", -1, 1),
        Among(u"ics", -1, 4),
        Among(u"l\u00F3gics", 88, 3),
        Among(u"uds", -1, 1),
        Among(u"nces", -1, 1),
        Among(u"ades", -1, 2),
        Among(u"ancies", -1, 1),
        Among(u"encies", -1, 1),
        Among(u"\u00E8ncies", -1, 1),
        Among(u"\u00EDcies", -1, 1),
        Among(u"logies", -1, 3),
        Among(u"inies", -1, 1),
        Among(u"\u00EDnies", -1, 1),
        Among(u"eries", -1, 1),
        Among(u"\u00E0ries", -1, 1),
        Among(u"at\u00F2ries", -1, 1),
        Among(u"bles", -1, 1),
        Among(u"ables", 103, 1),
        Among(u"ibles", 103, 1),
        Among(u"imes", -1, 1),
        Among(u"\u00EDssimes", 106, 1),
        Among(u"qu\u00EDssimes", 107, 5),
        Among(u"formes", -1, 1),
        Among(u"ismes", -1, 1),
        Among(u"ialismes", 110, 1),
        Among(u"ines", -1, 1),
        Among(u"eres", -1, 1),
        Among(u"ores", -1, 1),
        Among(u"dores", 114, 1),
        Among(u"idores", 115, 1),
        Among(u"dures", -1, 1),
        Among(u"eses", -1, 1),
        Among(u"oses", -1, 1),
        Among(u"asses", -1, 1),
        Among(u"ictes", -1, 1),
        Among(u"ites", -1, 1),
        Among(u"otes", -1, 1),
        Among(u"istes", -1, 1),
        Among(u"ialistes", 124, 1),
        Among(u"ionistes", 124, 1),
        Among(u"iques", -1, 4),
        Among(u"l\u00F3giques", 127, 3),
        Among(u"ives", -1, 1),
        Among(u"atives", 129, 1),
        Among(u"log\u00EDes", -1, 3),
        Among(u"alleng\u00FCes", -1, 1),
        Among(u"icis", -1, 1),
        Among(u"\u00EDcis", -1, 1),
        Among(u"logis", -1, 3),
        Among(u"aris", -1, 1),
        Among(u"toris", -1, 1),
        Among(u"ls", -1, 1),
        Among(u"als", 138, 1),
        Among(u"ells", 138, 1),
        Among(u"ims", -1, 1),
        Among(u"\u00EDssims", 141, 1),
        Among(u"qu\u00EDssims", 142, 5),
        Among(u"ions", -1, 1),
        Among(u"cions", 144, 1),
        Among(u"acions", 145, 2),
        Among(u"esos", -1, 1),
        Among(u"osos", -1, 1),
        Among(u"assos", -1, 1),
        Among(u"issos", -1, 1),
        Among(u"ers", -1, 1),
        Among(u"ors", -1, 1),
        Among(u"dors", 152, 1),
        Among(u"adors", 153, 1),
        Among(u"idors", 153, 1),
        Among(u"ats", -1, 1),
        Among(u"itats", 156, 1),
        Among(u"bilitats", 157, 1),
        Among(u"ivitats", 157, 1),
        Among(u"ativitats", 159, 1),
        Among(u"\u00EFtats", 156, 1),
        Among(u"ets", -1, 1),
        Among(u"ants", -1, 1),
        Among(u"ents", -1, 1),
        Among(u"ments", 164, 1),
        Among(u"aments", 165, 1),
        Among(u"ots", -1, 1),
        Among(u"uts", -1, 1),
        Among(u"ius", -1, 1),
        Among(u"trius", 169, 1),
        Among(u"atius", 169, 1),
        Among(u"\u00E8s", -1, 1),
        Among(u"\u00E9s", -1, 1),
        Among(u"\u00EDs", -1, 1),
        Among(u"d\u00EDs", 174, 1),
        Among(u"\u00F3s", -1, 1),
        Among(u"itat", -1, 1),
        Among(u"bilitat", 177, 1),
        Among(u"ivitat", 177, 1),
        Among(u"ativitat", 179, 1),
        Among(u"\u00EFtat", -1, 1),
        Among(u"et", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ent", -1, 1),
        Among(u"ient", 184, 1),
        Among(u"ment", 184, 1),
        Among(u"ament", 186, 1),
        Among(u"isament", 187, 1),
        Among(u"ot", -1, 1),
        Among(u"isseu", -1, 1),
        Among(u"\u00ECsseu", -1, 1),
        Among(u"\u00EDsseu", -1, 1),
        Among(u"triu", -1, 1),
        Among(u"\u00EDssiu", -1, 1),
        Among(u"atiu", -1, 1),
        Among(u"\u00F3", -1, 1),
        Among(u"i\u00F3", 196, 1),
        Among(u"ci\u00F3", 197, 1),
        Among(u"aci\u00F3", 198, 1)
    ]

    a_3 = [
        Among(u"aba", -1, 1),
        Among(u"esca", -1, 1),
        Among(u"isca", -1, 1),
        Among(u"\u00EFsca", -1, 1),
        Among(u"ada", -1, 1),
        Among(u"ida", -1, 1),
        Among(u"uda", -1, 1),
        Among(u"\u00EFda", -1, 1),
        Among(u"ia", -1, 1),
        Among(u"aria", 8, 1),
        Among(u"iria", 8, 1),
        Among(u"ara", -1, 1),
        Among(u"iera", -1, 1),
        Among(u"ira", -1, 1),
        Among(u"adora", -1, 1),
        Among(u"\u00EFra", -1, 1),
        Among(u"ava", -1, 1),
        Among(u"ixa", -1, 1),
        Among(u"itza", -1, 1),
        Among(u"\u00EDa", -1, 1),
        Among(u"ar\u00EDa", 19, 1),
        Among(u"er\u00EDa", 19, 1),
        Among(u"ir\u00EDa", 19, 1),
        Among(u"\u00EFa", -1, 1),
        Among(u"isc", -1, 1),
        Among(u"\u00EFsc", -1, 1),
        Among(u"ad", -1, 1),
        Among(u"ed", -1, 1),
        Among(u"id", -1, 1),
        Among(u"ie", -1, 1),
        Among(u"re", -1, 1),
        Among(u"dre", 30, 1),
        Among(u"ase", -1, 1),
        Among(u"iese", -1, 1),
        Among(u"aste", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ii", -1, 1),
        Among(u"ini", -1, 1),
        Among(u"esqui", -1, 1),
        Among(u"eixi", -1, 1),
        Among(u"itzi", -1, 1),
        Among(u"am", -1, 1),
        Among(u"em", -1, 1),
        Among(u"arem", 42, 1),
        Among(u"irem", 42, 1),
        Among(u"\u00E0rem", 42, 1),
        Among(u"\u00EDrem", 42, 1),
        Among(u"\u00E0ssem", 42, 1),
        Among(u"\u00E9ssem", 42, 1),
        Among(u"iguem", 42, 1),
        Among(u"\u00EFguem", 42, 1),
        Among(u"avem", 42, 1),
        Among(u"\u00E0vem", 42, 1),
        Among(u"\u00E1vem", 42, 1),
        Among(u"ir\u00ECem", 42, 1),
        Among(u"\u00EDem", 42, 1),
        Among(u"ar\u00EDem", 55, 1),
        Among(u"ir\u00EDem", 55, 1),
        Among(u"assim", -1, 1),
        Among(u"essim", -1, 1),
        Among(u"issim", -1, 1),
        Among(u"\u00E0ssim", -1, 1),
        Among(u"\u00E8ssim", -1, 1),
        Among(u"\u00E9ssim", -1, 1),
        Among(u"\u00EDssim", -1, 1),
        Among(u"\u00EFm", -1, 1),
        Among(u"an", -1, 1),
        Among(u"aban", 66, 1),
        Among(u"arian", 66, 1),
        Among(u"aran", 66, 1),
        Among(u"ieran", 66, 1),
        Among(u"iran", 66, 1),
        Among(u"\u00EDan", 66, 1),
        Among(u"ar\u00EDan", 72, 1),
        Among(u"er\u00EDan", 72, 1),
        Among(u"ir\u00EDan", 72, 1),
        Among(u"en", -1, 1),
        Among(u"ien", 76, 1),
        Among(u"arien", 77, 1),
        Among(u"irien", 77, 1),
        Among(u"aren", 76, 1),
        Among(u"eren", 76, 1),
        Among(u"iren", 76, 1),
        Among(u"\u00E0ren", 76, 1),
        Among(u"\u00EFren", 76, 1),
        Among(u"asen", 76, 1),
        Among(u"iesen", 76, 1),
        Among(u"assen", 76, 1),
        Among(u"essen", 76, 1),
        Among(u"issen", 76, 1),
        Among(u"\u00E9ssen", 76, 1),
        Among(u"\u00EFssen", 76, 1),
        Among(u"esquen", 76, 1),
        Among(u"isquen", 76, 1),
        Among(u"\u00EFsquen", 76, 1),
        Among(u"aven", 76, 1),
        Among(u"ixen", 76, 1),
        Among(u"eixen", 96, 1),
        Among(u"\u00EFxen", 76, 1),
        Among(u"\u00EFen", 76, 1),
        Among(u"in", -1, 1),
        Among(u"inin", 100, 1),
        Among(u"sin", 100, 1),
        Among(u"isin", 102, 1),
        Among(u"assin", 102, 1),
        Among(u"essin", 102, 1),
        Among(u"issin", 102, 1),
        Among(u"\u00EFssin", 102, 1),
        Among(u"esquin", 100, 1),
        Among(u"eixin", 100, 1),
        Among(u"aron", -1, 1),
        Among(u"ieron", -1, 1),
        Among(u"ar\u00E1n", -1, 1),
        Among(u"er\u00E1n", -1, 1),
        Among(u"ir\u00E1n", -1, 1),
        Among(u"i\u00EFn", -1, 1),
        Among(u"ado", -1, 1),
        Among(u"ido", -1, 1),
        Among(u"ando", -1, 2),
        Among(u"iendo", -1, 1),
        Among(u"io", -1, 1),
        Among(u"ixo", -1, 1),
        Among(u"eixo", 121, 1),
        Among(u"\u00EFxo", -1, 1),
        Among(u"itzo", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"tzar", 125, 1),
        Among(u"er", -1, 1),
        Among(u"eixer", 127, 1),
        Among(u"ir", -1, 1),
        Among(u"ador", -1, 1),
        Among(u"as", -1, 1),
        Among(u"abas", 131, 1),
        Among(u"adas", 131, 1),
        Among(u"idas", 131, 1),
        Among(u"aras", 131, 1),
        Among(u"ieras", 131, 1),
        Among(u"\u00EDas", 131, 1),
        Among(u"ar\u00EDas", 137, 1),
        Among(u"er\u00EDas", 137, 1),
        Among(u"ir\u00EDas", 137, 1),
        Among(u"ids", -1, 1),
        Among(u"es", -1, 1),
        Among(u"ades", 142, 1),
        Among(u"ides", 142, 1),
        Among(u"udes", 142, 1),
        Among(u"\u00EFdes", 142, 1),
        Among(u"atges", 142, 1),
        Among(u"ies", 142, 1),
        Among(u"aries", 148, 1),
        Among(u"iries", 148, 1),
        Among(u"ares", 142, 1),
        Among(u"ires", 142, 1),
        Among(u"adores", 142, 1),
        Among(u"\u00EFres", 142, 1),
        Among(u"ases", 142, 1),
        Among(u"ieses", 142, 1),
        Among(u"asses", 142, 1),
        Among(u"esses", 142, 1),
        Among(u"isses", 142, 1),
        Among(u"\u00EFsses", 142, 1),
        Among(u"ques", 142, 1),
        Among(u"esques", 161, 1),
        Among(u"\u00EFsques", 161, 1),
        Among(u"aves", 142, 1),
        Among(u"ixes", 142, 1),
        Among(u"eixes", 165, 1),
        Among(u"\u00EFxes", 142, 1),
        Among(u"\u00EFes", 142, 1),
        Among(u"abais", -1, 1),
        Among(u"arais", -1, 1),
        Among(u"ierais", -1, 1),
        Among(u"\u00EDais", -1, 1),
        Among(u"ar\u00EDais", 172, 1),
        Among(u"er\u00EDais", 172, 1),
        Among(u"ir\u00EDais", 172, 1),
        Among(u"aseis", -1, 1),
        Among(u"ieseis", -1, 1),
        Among(u"asteis", -1, 1),
        Among(u"isteis", -1, 1),
        Among(u"inis", -1, 1),
        Among(u"sis", -1, 1),
        Among(u"isis", 181, 1),
        Among(u"assis", 181, 1),
        Among(u"essis", 181, 1),
        Among(u"issis", 181, 1),
        Among(u"\u00EFssis", 181, 1),
        Among(u"esquis", -1, 1),
        Among(u"eixis", -1, 1),
        Among(u"itzis", -1, 1),
        Among(u"\u00E1is", -1, 1),
        Among(u"ar\u00E9is", -1, 1),
        Among(u"er\u00E9is", -1, 1),
        Among(u"ir\u00E9is", -1, 1),
        Among(u"ams", -1, 1),
        Among(u"ados", -1, 1),
        Among(u"idos", -1, 1),
        Among(u"amos", -1, 1),
        Among(u"\u00E1bamos", 197, 1),
        Among(u"\u00E1ramos", 197, 1),
        Among(u"i\u00E9ramos", 197, 1),
        Among(u"\u00EDamos", 197, 1),
        Among(u"ar\u00EDamos", 201, 1),
        Among(u"er\u00EDamos", 201, 1),
        Among(u"ir\u00EDamos", 201, 1),
        Among(u"aremos", -1, 1),
        Among(u"eremos", -1, 1),
        Among(u"iremos", -1, 1),
        Among(u"\u00E1semos", -1, 1),
        Among(u"i\u00E9semos", -1, 1),
        Among(u"imos", -1, 1),
        Among(u"adors", -1, 1),
        Among(u"ass", -1, 1),
        Among(u"erass", 212, 1),
        Among(u"ess", -1, 1),
        Among(u"ats", -1, 1),
        Among(u"its", -1, 1),
        Among(u"ents", -1, 1),
        Among(u"\u00E0s", -1, 1),
        Among(u"ar\u00E0s", 218, 1),
        Among(u"ir\u00E0s", 218, 1),
        Among(u"ar\u00E1s", -1, 1),
        Among(u"er\u00E1s", -1, 1),
        Among(u"ir\u00E1s", -1, 1),
        Among(u"\u00E9s", -1, 1),
        Among(u"ar\u00E9s", 224, 1),
        Among(u"\u00EDs", -1, 1),
        Among(u"i\u00EFs", -1, 1),
        Among(u"at", -1, 1),
        Among(u"it", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ent", -1, 1),
        Among(u"int", -1, 1),
        Among(u"ut", -1, 1),
        Among(u"\u00EFt", -1, 1),
        Among(u"au", -1, 1),
        Among(u"erau", 235, 1),
        Among(u"ieu", -1, 1),
        Among(u"ineu", -1, 1),
        Among(u"areu", -1, 1),
        Among(u"ireu", -1, 1),
        Among(u"\u00E0reu", -1, 1),
        Among(u"\u00EDreu", -1, 1),
        Among(u"asseu", -1, 1),
        Among(u"esseu", -1, 1),
        Among(u"eresseu", 244, 1),
        Among(u"\u00E0sseu", -1, 1),
        Among(u"\u00E9sseu", -1, 1),
        Among(u"igueu", -1, 1),
        Among(u"\u00EFgueu", -1, 1),
        Among(u"\u00E0veu", -1, 1),
        Among(u"\u00E1veu", -1, 1),
        Among(u"itzeu", -1, 1),
        Among(u"\u00ECeu", -1, 1),
        Among(u"ir\u00ECeu", 253, 1),
        Among(u"\u00EDeu", -1, 1),
        Among(u"ar\u00EDeu", 255, 1),
        Among(u"ir\u00EDeu", 255, 1),
        Among(u"assiu", -1, 1),
        Among(u"issiu", -1, 1),
        Among(u"\u00E0ssiu", -1, 1),
        Among(u"\u00E8ssiu", -1, 1),
        Among(u"\u00E9ssiu", -1, 1),
        Among(u"\u00EDssiu", -1, 1),
        Among(u"\u00EFu", -1, 1),
        Among(u"ix", -1, 1),
        Among(u"eix", 265, 1),
        Among(u"\u00EFx", -1, 1),
        Among(u"itz", -1, 1),
        Among(u"i\u00E0", -1, 1),
        Among(u"ar\u00E0", -1, 1),
        Among(u"ir\u00E0", -1, 1),
        Among(u"itz\u00E0", -1, 1),
        Among(u"ar\u00E1", -1, 1),
        Among(u"er\u00E1", -1, 1),
        Among(u"ir\u00E1", -1, 1),
        Among(u"ir\u00E8", -1, 1),
        Among(u"ar\u00E9", -1, 1),
        Among(u"er\u00E9", -1, 1),
        Among(u"ir\u00E9", -1, 1),
        Among(u"\u00ED", -1, 1),
        Among(u"i\u00EF", -1, 1),
        Among(u"i\u00F3", -1, 1)
    ]

    a_4 = [
        Among(u"a", -1, 1),
        Among(u"e", -1, 1),
        Among(u"i", -1, 1),
        Among(u"\u00EFn", -1, 1),
        Among(u"o", -1, 1),
        Among(u"ir", -1, 1),
        Among(u"s", -1, 1),
        Among(u"is", 6, 1),
        Among(u"os", 6, 1),
        Among(u"\u00EFs", 6, 1),
        Among(u"it", -1, 1),
        Among(u"eu", -1, 1),
        Among(u"iu", -1, 1),
        Among(u"iqu", -1, 2),
        Among(u"itz", -1, 1),
        Among(u"\u00E0", -1, 1),
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 1),
        Among(u"\u00EC", -1, 1),
        Among(u"\u00ED", -1, 1),
        Among(u"\u00EF", -1, 1),
        Among(u"\u00F3", -1, 1)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128, 129, 81, 6, 10]

    I_p2 = 0
    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            if not self.go_out_grouping(CatalanStemmer.g_v, 97, 252):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(CatalanStemmer.g_v, 97, 252):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(CatalanStemmer.g_v, 97, 252):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(CatalanStemmer.g_v, 97, 252):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_cleaning(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(CatalanStemmer.a_0)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"e"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"u"):
                        return False
                elif among_var == 6:
                    if not self.slice_from(u"."):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_attached_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(CatalanStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.slice_del():
            return False

        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 3:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"log"):
                return False
        elif among_var == 4:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"ic"):
                return False
        else:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"c"):
                return False
        return True

    def __r_verb_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"ic"):
                return False
        return True

    def _stem(self):
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_attached_pronoun()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    if not self.__r_standard_suffix():
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                if not self.__r_verb_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        v_5 = self.limit - self.cursor
        self.__r_residual_suffix()
        self.cursor = self.limit - v_5
        self.cursor = self.limit_backward
        v_6 = self.cursor
        self.__r_cleaning()
        self.cursor = v_6
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class YiddishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"\u05D5\u05D5", -1, 1),
        Among(u"\u05D5\u05D9", -1, 2),
        Among(u"\u05D9\u05D9", -1, 3),
        Among(u"\u05DA", -1, 4),
        Among(u"\u05DD", -1, 5),
        Among(u"\u05DF", -1, 6),
        Among(u"\u05E3", -1, 7),
        Among(u"\u05E5", -1, 8)
    ]

    a_1 = [
        Among(u"\u05D0\u05D3\u05D5\u05E8\u05DB", -1, 1),
        Among(u"\u05D0\u05D4\u05D9\u05E0", -1, 1),
        Among(u"\u05D0\u05D4\u05E2\u05E8", -1, 1),
        Among(u"\u05D0\u05D4\u05F2\u05DE", -1, 1),
        Among(u"\u05D0\u05D5\u05DE", -1, 1),
        Among(u"\u05D0\u05D5\u05E0\u05D8\u05E2\u05E8", -1, 1),
        Among(u"\u05D0\u05D9\u05D1\u05E2\u05E8", -1, 1),
        Among(u"\u05D0\u05E0", -1, 1),
        Among(u"\u05D0\u05E0\u05D8", 7, 1),
        Among(u"\u05D0\u05E0\u05D8\u05E7\u05E2\u05D2\u05E0", 8, 1),
        Among(u"\u05D0\u05E0\u05D9\u05D3\u05E2\u05E8", 7, 1),
        Among(u"\u05D0\u05E4", -1, 1),
        Among(u"\u05D0\u05E4\u05D9\u05E8", 11, 1),
        Among(u"\u05D0\u05E7\u05E2\u05D2\u05E0", -1, 1),
        Among(u"\u05D0\u05E8\u05D0\u05E4", -1, 1),
        Among(u"\u05D0\u05E8\u05D5\u05DE", -1, 1),
        Among(u"\u05D0\u05E8\u05D5\u05E0\u05D8\u05E2\u05E8", -1, 1),
        Among(u"\u05D0\u05E8\u05D9\u05D1\u05E2\u05E8", -1, 1),
        Among(u"\u05D0\u05E8\u05F1\u05E1", -1, 1),
        Among(u"\u05D0\u05E8\u05F1\u05E4", -1, 1),
        Among(u"\u05D0\u05E8\u05F2\u05E0", -1, 1),
        Among(u"\u05D0\u05F0\u05E2\u05E7", -1, 1),
        Among(u"\u05D0\u05F1\u05E1", -1, 1),
        Among(u"\u05D0\u05F1\u05E4", -1, 1),
        Among(u"\u05D0\u05F2\u05E0", -1, 1),
        Among(u"\u05D1\u05D0", -1, 1),
        Among(u"\u05D1\u05F2", -1, 1),
        Among(u"\u05D3\u05D5\u05E8\u05DB", -1, 1),
        Among(u"\u05D3\u05E2\u05E8", -1, 1),
        Among(u"\u05DE\u05D9\u05D8", -1, 1),
        Among(u"\u05E0\u05D0\u05DB", -1, 1),
        Among(u"\u05E4\u05D0\u05E8", -1, 1),
        Among(u"\u05E4\u05D0\u05E8\u05D1\u05F2", 31, 1),
        Among(u"\u05E4\u05D0\u05E8\u05F1\u05E1", 31, 1),
        Among(u"\u05E4\u05D5\u05E0\u05D0\u05E0\u05D3\u05E2\u05E8", -1, 1),
        Among(u"\u05E6\u05D5", -1, 1),
        Among(u"\u05E6\u05D5\u05D6\u05D0\u05DE\u05E2\u05E0", 35, 1),
        Among(u"\u05E6\u05D5\u05E0\u05F1\u05E4", 35, 1),
        Among(u"\u05E6\u05D5\u05E8\u05D9\u05E7", 35, 1),
        Among(u"\u05E6\u05E2", -1, 1)
    ]

    a_2 = [
        Among(u"\u05D3\u05D6\u05E9", -1, -1),
        Among(u"\u05E9\u05D8\u05E8", -1, -1),
        Among(u"\u05E9\u05D8\u05E9", -1, -1),
        Among(u"\u05E9\u05E4\u05E8", -1, -1)
    ]

    a_3 = [
        Among(u"\u05E7\u05DC\u05D9\u05D1", -1, 9),
        Among(u"\u05E8\u05D9\u05D1", -1, 10),
        Among(u"\u05D8\u05E8\u05D9\u05D1", 1, 7),
        Among(u"\u05E9\u05E8\u05D9\u05D1", 1, 15),
        Among(u"\u05D4\u05F1\u05D1", -1, 23),
        Among(u"\u05E9\u05F0\u05D9\u05D2", -1, 12),
        Among(u"\u05D2\u05D0\u05E0\u05D2", -1, 1),
        Among(u"\u05D6\u05D5\u05E0\u05D2", -1, 18),
        Among(u"\u05E9\u05DC\u05D5\u05E0\u05D2", -1, 21),
        Among(u"\u05E6\u05F0\u05D5\u05E0\u05D2", -1, 20),
        Among(u"\u05D1\u05F1\u05D2", -1, 22),
        Among(u"\u05D1\u05D5\u05E0\u05D3", -1, 16),
        Among(u"\u05F0\u05D9\u05D6", -1, 6),
        Among(u"\u05D1\u05D9\u05D8", -1, 4),
        Among(u"\u05DC\u05D9\u05D8", -1, 8),
        Among(u"\u05DE\u05D9\u05D8", -1, 3),
        Among(u"\u05E9\u05E0\u05D9\u05D8", -1, 14),
        Among(u"\u05E0\u05D5\u05DE", -1, 2),
        Among(u"\u05E9\u05D8\u05D0\u05E0", -1, 25),
        Among(u"\u05D1\u05D9\u05E1", -1, 5),
        Among(u"\u05E9\u05DE\u05D9\u05E1", -1, 13),
        Among(u"\u05E8\u05D9\u05E1", -1, 11),
        Among(u"\u05D8\u05E8\u05D5\u05E0\u05E7", -1, 19),
        Among(u"\u05E4\u05D0\u05E8\u05DC\u05F1\u05E8", -1, 24),
        Among(u"\u05E9\u05F0\u05F1\u05E8", -1, 26),
        Among(u"\u05F0\u05D5\u05D8\u05E9", -1, 17)
    ]

    a_4 = [
        Among(u"\u05D5\u05E0\u05D2", -1, 1),
        Among(u"\u05E1\u05D8\u05D5", -1, 1),
        Among(u"\u05D8", -1, 1),
        Among(u"\u05D1\u05E8\u05D0\u05DB\u05D8", 2, 31),
        Among(u"\u05E1\u05D8", 2, 1),
        Among(u"\u05D9\u05E1\u05D8", 4, 33),
        Among(u"\u05E2\u05D8", 2, 1),
        Among(u"\u05E9\u05D0\u05E4\u05D8", 2, 1),
        Among(u"\u05D4\u05F2\u05D8", 2, 1),
        Among(u"\u05E7\u05F2\u05D8", 2, 1),
        Among(u"\u05D9\u05E7\u05F2\u05D8", 9, 1),
        Among(u"\u05DC\u05E2\u05DB", -1, 1),
        Among(u"\u05E2\u05DC\u05E2\u05DB", 11, 1),
        Among(u"\u05D9\u05D6\u05DE", -1, 1),
        Among(u"\u05D9\u05DE", -1, 1),
        Among(u"\u05E2\u05DE", -1, 1),
        Among(u"\u05E2\u05E0\u05E2\u05DE", 15, 3),
        Among(u"\u05D8\u05E2\u05E0\u05E2\u05DE", 16, 4),
        Among(u"\u05E0", -1, 1),
        Among(u"\u05E7\u05DC\u05D9\u05D1\u05E0", 18, 14),
        Among(u"\u05E8\u05D9\u05D1\u05E0", 18, 15),
        Among(u"\u05D8\u05E8\u05D9\u05D1\u05E0", 20, 12),
        Among(u"\u05E9\u05E8\u05D9\u05D1\u05E0", 20, 7),
        Among(u"\u05D4\u05F1\u05D1\u05E0", 18, 27),
        Among(u"\u05E9\u05F0\u05D9\u05D2\u05E0", 18, 17),
        Among(u"\u05D6\u05D5\u05E0\u05D2\u05E0", 18, 22),
        Among(u"\u05E9\u05DC\u05D5\u05E0\u05D2\u05E0", 18, 25),
        Among(u"\u05E6\u05F0\u05D5\u05E0\u05D2\u05E0", 18, 24),
        Among(u"\u05D1\u05F1\u05D2\u05E0", 18, 26),
        Among(u"\u05D1\u05D5\u05E0\u05D3\u05E0", 18, 20),
        Among(u"\u05F0\u05D9\u05D6\u05E0", 18, 11),
        Among(u"\u05D8\u05E0", 18, 4),
        Among(u"GE\u05D1\u05D9\u05D8\u05E0", 31, 9),
        Among(u"GE\u05DC\u05D9\u05D8\u05E0", 31, 13),
        Among(u"GE\u05DE\u05D9\u05D8\u05E0", 31, 8),
        Among(u"\u05E9\u05E0\u05D9\u05D8\u05E0", 31, 19),
        Among(u"\u05E1\u05D8\u05E0", 31, 1),
        Among(u"\u05D9\u05E1\u05D8\u05E0", 36, 1),
        Among(u"\u05E2\u05D8\u05E0", 31, 1),
        Among(u"GE\u05D1\u05D9\u05E1\u05E0", 18, 10),
        Among(u"\u05E9\u05DE\u05D9\u05E1\u05E0", 18, 18),
        Among(u"GE\u05E8\u05D9\u05E1\u05E0", 18, 16),
        Among(u"\u05E2\u05E0", 18, 1),
        Among(u"\u05D2\u05D0\u05E0\u05D2\u05E2\u05E0", 42, 5),
        Among(u"\u05E2\u05DC\u05E2\u05E0", 42, 1),
        Among(u"\u05E0\u05D5\u05DE\u05E2\u05E0", 42, 6),
        Among(u"\u05D9\u05D6\u05DE\u05E2\u05E0", 42, 1),
        Among(u"\u05E9\u05D8\u05D0\u05E0\u05E2\u05E0", 42, 29),
        Among(u"\u05D8\u05E8\u05D5\u05E0\u05E7\u05E0", 18, 23),
        Among(u"\u05E4\u05D0\u05E8\u05DC\u05F1\u05E8\u05E0", 18, 28),
        Among(u"\u05E9\u05F0\u05F1\u05E8\u05E0", 18, 30),
        Among(u"\u05F0\u05D5\u05D8\u05E9\u05E0", 18, 21),
        Among(u"\u05D2\u05F2\u05E0", 18, 5),
        Among(u"\u05E1", -1, 1),
        Among(u"\u05D8\u05E1", 53, 4),
        Among(u"\u05E2\u05D8\u05E1", 54, 1),
        Among(u"\u05E0\u05E1", 53, 1),
        Among(u"\u05D8\u05E0\u05E1", 56, 4),
        Among(u"\u05E2\u05E0\u05E1", 56, 3),
        Among(u"\u05E2\u05E1", 53, 1),
        Among(u"\u05D9\u05E2\u05E1", 59, 2),
        Among(u"\u05E2\u05DC\u05E2\u05E1", 59, 1),
        Among(u"\u05E2\u05E8\u05E1", 53, 1),
        Among(u"\u05E2\u05E0\u05E2\u05E8\u05E1", 62, 1),
        Among(u"\u05E2", -1, 1),
        Among(u"\u05D8\u05E2", 64, 4),
        Among(u"\u05E1\u05D8\u05E2", 65, 1),
        Among(u"\u05E2\u05D8\u05E2", 65, 1),
        Among(u"\u05D9\u05E2", 64, -1),
        Among(u"\u05E2\u05DC\u05E2", 64, 1),
        Among(u"\u05E2\u05E0\u05E2", 64, 3),
        Among(u"\u05D8\u05E2\u05E0\u05E2", 70, 4),
        Among(u"\u05E2\u05E8", -1, 1),
        Among(u"\u05D8\u05E2\u05E8", 72, 4),
        Among(u"\u05E1\u05D8\u05E2\u05E8", 73, 1),
        Among(u"\u05E2\u05D8\u05E2\u05E8", 73, 1),
        Among(u"\u05E2\u05E0\u05E2\u05E8", 72, 3),
        Among(u"\u05D8\u05E2\u05E0\u05E2\u05E8", 76, 4),
        Among(u"\u05D5\u05EA", -1, 32)
    ]

    a_5 = [
        Among(u"\u05D5\u05E0\u05D2", -1, 1),
        Among(u"\u05E9\u05D0\u05E4\u05D8", -1, 1),
        Among(u"\u05D4\u05F2\u05D8", -1, 1),
        Among(u"\u05E7\u05F2\u05D8", -1, 1),
        Among(u"\u05D9\u05E7\u05F2\u05D8", 3, 1),
        Among(u"\u05DC", -1, 2)
    ]

    a_6 = [
        Among(u"\u05D9\u05D2", -1, 1),
        Among(u"\u05D9\u05E7", -1, 1),
        Among(u"\u05D3\u05D9\u05E7", 1, 1),
        Among(u"\u05E0\u05D3\u05D9\u05E7", 2, 1),
        Among(u"\u05E2\u05E0\u05D3\u05D9\u05E7", 3, 1),
        Among(u"\u05D1\u05DC\u05D9\u05E7", 1, -1),
        Among(u"\u05D2\u05DC\u05D9\u05E7", 1, -1),
        Among(u"\u05E0\u05D9\u05E7", 1, 1),
        Among(u"\u05D9\u05E9", -1, 1)
    ]

    g_niked = [255, 155, 6]

    g_vowel = [33, 2, 4, 0, 6]

    g_consonant = [239, 254, 253, 131]

    I_x = 0
    I_p1 = 0

    def __r_prelude(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    try:
                        while True:
                            v_3 = self.cursor
                            try:
                                self.bra = self.cursor
                                among_var = self.find_among(YiddishStemmer.a_0)
                                if among_var == 0:
                                    raise lab3()
                                self.ket = self.cursor
                                if among_var == 1:
                                    v_4 = self.cursor
                                    try:
                                        if not self.eq_s(u"\u05BC"):
                                            raise lab4()
                                        raise lab3()
                                    except lab4: pass
                                    self.cursor = v_4
                                    if not self.slice_from(u"\u05F0"):
                                        return False
                                elif among_var == 2:
                                    v_5 = self.cursor
                                    try:
                                        if not self.eq_s(u"\u05B4"):
                                            raise lab5()
                                        raise lab3()
                                    except lab5: pass
                                    self.cursor = v_5
                                    if not self.slice_from(u"\u05F1"):
                                        return False
                                elif among_var == 3:
                                    v_6 = self.cursor
                                    try:
                                        if not self.eq_s(u"\u05B4"):
                                            raise lab6()
                                        raise lab3()
                                    except lab6: pass
                                    self.cursor = v_6
                                    if not self.slice_from(u"\u05F2"):
                                        return False
                                elif among_var == 4:
                                    if not self.slice_from(u"\u05DB"):
                                        return False
                                elif among_var == 5:
                                    if not self.slice_from(u"\u05DE"):
                                        return False
                                elif among_var == 6:
                                    if not self.slice_from(u"\u05E0"):
                                        return False
                                elif among_var == 7:
                                    if not self.slice_from(u"\u05E4"):
                                        return False
                                else:
                                    if not self.slice_from(u"\u05E6"):
                                        return False
                                self.cursor = v_3
                                raise lab2()
                            except lab3: pass
                            self.cursor = v_3
                            if self.cursor >= self.limit:
                                raise lab1()
                            self.cursor += 1
                    except lab2: pass
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        v_7 = self.cursor
        try:
            while True:
                v_8 = self.cursor
                try:
                    try:
                        while True:
                            v_9 = self.cursor
                            try:
                                self.bra = self.cursor
                                if not self.in_grouping(YiddishStemmer.g_niked, 1456, 1474):
                                    raise lab10()
                                self.ket = self.cursor
                                if not self.slice_del():
                                    return False

                                self.cursor = v_9
                                raise lab9()
                            except lab10: pass
                            self.cursor = v_9
                            if self.cursor >= self.limit:
                                raise lab8()
                            self.cursor += 1
                    except lab9: pass
                    continue
                except lab8: pass
                self.cursor = v_8
                break
        except lab7: pass
        self.cursor = v_7
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            if not self.eq_s(u"\u05D2\u05E2"):
                self.cursor = v_1
                raise lab0()
            self.ket = self.cursor
            v_2 = self.cursor
            try:
                try:
                    v_3 = self.cursor
                    try:
                        if not self.eq_s(u"\u05DC\u05D8"):
                            raise lab3()
                        raise lab2()
                    except lab3: pass
                    self.cursor = v_3
                    if not self.eq_s(u"\u05D1\u05E0"):
                        raise lab1()
                except lab2: pass
                self.cursor = v_1
                raise lab0()
            except lab1: pass
            self.cursor = v_2
            if not self.slice_from(u"GE"):
                return False
        except lab0: pass
        v_4 = self.cursor
        try:
            if self.find_among(YiddishStemmer.a_1) == 0:
                self.cursor = v_4
                raise lab4()
            try:
                v_5 = self.cursor
                try:
                    v_6 = self.cursor
                    try:
                        v_7 = self.cursor
                        try:
                            if not self.eq_s(u"\u05E6\u05D5\u05D2\u05E0"):
                                raise lab8()
                            raise lab7()
                        except lab8: pass
                        self.cursor = v_7
                        try:
                            if not self.eq_s(u"\u05E6\u05D5\u05E7\u05D8"):
                                raise lab9()
                            raise lab7()
                        except lab9: pass
                        self.cursor = v_7
                        if not self.eq_s(u"\u05E6\u05D5\u05E7\u05E0"):
                            raise lab6()
                    except lab7: pass
                    if self.cursor < self.limit:
                        raise lab6()
                    self.cursor = v_6
                    raise lab5()
                except lab6: pass
                self.cursor = v_5
                try:
                    v_8 = self.cursor
                    if not self.eq_s(u"\u05D2\u05E2\u05D1\u05E0"):
                        raise lab10()
                    self.cursor = v_8
                    raise lab5()
                except lab10: pass
                self.cursor = v_5
                try:
                    self.bra = self.cursor
                    if not self.eq_s(u"\u05D2\u05E2"):
                        raise lab11()
                    self.ket = self.cursor
                    if not self.slice_from(u"GE"):
                        return False
                    raise lab5()
                except lab11: pass
                self.cursor = v_5
                self.bra = self.cursor
                if not self.eq_s(u"\u05E6\u05D5"):
                    self.cursor = v_4
                    raise lab4()
                self.ket = self.cursor
                if not self.slice_from(u"TSU"):
                    return False
            except lab5: pass
        except lab4: pass
        v_9 = self.cursor
        c = self.cursor + 3
        if c > self.limit:
            return False
        self.cursor = c
        self.I_x = self.cursor
        self.cursor = v_9
        v_10 = self.cursor
        try:
            if self.find_among(YiddishStemmer.a_2) == 0:
                self.cursor = v_10
                raise lab12()
        except lab12: pass
        v_11 = self.cursor
        try:
            if not self.in_grouping(YiddishStemmer.g_consonant, 1489, 1520):
                raise lab13()
            if not self.in_grouping(YiddishStemmer.g_consonant, 1489, 1520):
                raise lab13()
            if not self.in_grouping(YiddishStemmer.g_consonant, 1489, 1520):
                raise lab13()
            self.I_p1 = self.cursor
            return False
        except lab13: pass
        self.cursor = v_11
        if not self.go_out_grouping(YiddishStemmer.g_vowel, 1488, 1522):
            return False
        while True:
            try:
                if not self.in_grouping(YiddishStemmer.g_vowel, 1488, 1522):
                    raise lab14()
                continue
            except lab14: pass
            break
        self.I_p1 = self.cursor
        try:
            if not self.I_p1 < self.I_x:
                raise lab15()
            self.I_p1 = self.I_x
        except lab15: pass
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R1plus3(self):
        if not self.I_p1 <= (self.cursor + 3):
            return False
        return True

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_4)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab0()
                if not self.slice_del():
                    return False

            elif among_var == 2:
                if not self.__r_R1():
                    raise lab0()
                if not self.slice_from(u"\u05D9\u05E2"):
                    return False
            elif among_var == 3:
                if not self.__r_R1():
                    raise lab0()
                if not self.slice_del():
                    return False

                self.ket = self.cursor
                among_var = self.find_among_b(YiddishStemmer.a_3)
                if among_var == 0:
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"\u05D2\u05F2"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"\u05E0\u05E2\u05DE"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"\u05DE\u05F2\u05D3"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"\u05D1\u05F2\u05D8"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"\u05D1\u05F2\u05E1"):
                        return False
                elif among_var == 6:
                    if not self.slice_from(u"\u05F0\u05F2\u05D6"):
                        return False
                elif among_var == 7:
                    if not self.slice_from(u"\u05D8\u05E8\u05F2\u05D1"):
                        return False
                elif among_var == 8:
                    if not self.slice_from(u"\u05DC\u05F2\u05D8"):
                        return False
                elif among_var == 9:
                    if not self.slice_from(u"\u05E7\u05DC\u05F2\u05D1"):
                        return False
                elif among_var == 10:
                    if not self.slice_from(u"\u05E8\u05F2\u05D1"):
                        return False
                elif among_var == 11:
                    if not self.slice_from(u"\u05E8\u05F2\u05E1"):
                        return False
                elif among_var == 12:
                    if not self.slice_from(u"\u05E9\u05F0\u05F2\u05D2"):
                        return False
                elif among_var == 13:
                    if not self.slice_from(u"\u05E9\u05DE\u05F2\u05E1"):
                        return False
                elif among_var == 14:
                    if not self.slice_from(u"\u05E9\u05E0\u05F2\u05D3"):
                        return False
                elif among_var == 15:
                    if not self.slice_from(u"\u05E9\u05E8\u05F2\u05D1"):
                        return False
                elif among_var == 16:
                    if not self.slice_from(u"\u05D1\u05D9\u05E0\u05D3"):
                        return False
                elif among_var == 17:
                    if not self.slice_from(u"\u05F0\u05D9\u05D8\u05E9"):
                        return False
                elif among_var == 18:
                    if not self.slice_from(u"\u05D6\u05D9\u05E0\u05D2"):
                        return False
                elif among_var == 19:
                    if not self.slice_from(u"\u05D8\u05E8\u05D9\u05E0\u05E7"):
                        return False
                elif among_var == 20:
                    if not self.slice_from(u"\u05E6\u05F0\u05D9\u05E0\u05D2"):
                        return False
                elif among_var == 21:
                    if not self.slice_from(u"\u05E9\u05DC\u05D9\u05E0\u05D2"):
                        return False
                elif among_var == 22:
                    if not self.slice_from(u"\u05D1\u05F2\u05D2"):
                        return False
                elif among_var == 23:
                    if not self.slice_from(u"\u05D4\u05F2\u05D1"):
                        return False
                elif among_var == 24:
                    if not self.slice_from(u"\u05E4\u05D0\u05E8\u05DC\u05D9\u05E8"):
                        return False
                elif among_var == 25:
                    if not self.slice_from(u"\u05E9\u05D8\u05F2"):
                        return False
                else:
                    if not self.slice_from(u"\u05E9\u05F0\u05E2\u05E8"):
                        return False
            elif among_var == 4:
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_R1():
                            raise lab2()
                        if not self.slice_del():
                            return False

                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_2
                    if not self.slice_from(u"\u05D8"):
                        return False
                except lab1: pass
                self.ket = self.cursor
                if not self.eq_s_b(u"\u05D1\u05E8\u05D0\u05DB"):
                    raise lab0()
                v_3 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"\u05D2\u05E2"):
                        self.cursor = self.limit - v_3
                        raise lab3()
                except lab3: pass
                self.bra = self.cursor
                if not self.slice_from(u"\u05D1\u05E8\u05E2\u05E0\u05D2"):
                    return False
            elif among_var == 5:
                if not self.slice_from(u"\u05D2\u05F2"):
                    return False
            elif among_var == 6:
                if not self.slice_from(u"\u05E0\u05E2\u05DE"):
                    return False
            elif among_var == 7:
                if not self.slice_from(u"\u05E9\u05E8\u05F2\u05D1"):
                    return False
            elif among_var == 8:
                if not self.slice_from(u"\u05DE\u05F2\u05D3"):
                    return False
            elif among_var == 9:
                if not self.slice_from(u"\u05D1\u05F2\u05D8"):
                    return False
            elif among_var == 10:
                if not self.slice_from(u"\u05D1\u05F2\u05E1"):
                    return False
            elif among_var == 11:
                if not self.slice_from(u"\u05F0\u05F2\u05D6"):
                    return False
            elif among_var == 12:
                if not self.slice_from(u"\u05D8\u05E8\u05F2\u05D1"):
                    return False
            elif among_var == 13:
                if not self.slice_from(u"\u05DC\u05F2\u05D8"):
                    return False
            elif among_var == 14:
                if not self.slice_from(u"\u05E7\u05DC\u05F2\u05D1"):
                    return False
            elif among_var == 15:
                if not self.slice_from(u"\u05E8\u05F2\u05D1"):
                    return False
            elif among_var == 16:
                if not self.slice_from(u"\u05E8\u05F2\u05E1"):
                    return False
            elif among_var == 17:
                if not self.slice_from(u"\u05E9\u05F0\u05F2\u05D2"):
                    return False
            elif among_var == 18:
                if not self.slice_from(u"\u05E9\u05DE\u05F2\u05E1"):
                    return False
            elif among_var == 19:
                if not self.slice_from(u"\u05E9\u05E0\u05F2\u05D3"):
                    return False
            elif among_var == 20:
                if not self.slice_from(u"\u05D1\u05D9\u05E0\u05D3"):
                    return False
            elif among_var == 21:
                if not self.slice_from(u"\u05F0\u05D9\u05D8\u05E9"):
                    return False
            elif among_var == 22:
                if not self.slice_from(u"\u05D6\u05D9\u05E0\u05D2"):
                    return False
            elif among_var == 23:
                if not self.slice_from(u"\u05D8\u05E8\u05D9\u05E0\u05E7"):
                    return False
            elif among_var == 24:
                if not self.slice_from(u"\u05E6\u05F0\u05D9\u05E0\u05D2"):
                    return False
            elif among_var == 25:
                if not self.slice_from(u"\u05E9\u05DC\u05D9\u05E0\u05D2"):
                    return False
            elif among_var == 26:
                if not self.slice_from(u"\u05D1\u05F2\u05D2"):
                    return False
            elif among_var == 27:
                if not self.slice_from(u"\u05D4\u05F2\u05D1"):
                    return False
            elif among_var == 28:
                if not self.slice_from(u"\u05E4\u05D0\u05E8\u05DC\u05D9\u05E8"):
                    return False
            elif among_var == 29:
                if not self.slice_from(u"\u05E9\u05D8\u05F2"):
                    return False
            elif among_var == 30:
                if not self.slice_from(u"\u05E9\u05F0\u05E2\u05E8"):
                    return False
            elif among_var == 31:
                if not self.slice_from(u"\u05D1\u05E8\u05E2\u05E0\u05D2"):
                    return False
            elif among_var == 32:
                if not self.__r_R1():
                    raise lab0()
                if not self.slice_from(u"\u05D4"):
                    return False
            elif among_var == 33:
                try:
                    v_4 = self.limit - self.cursor
                    try:
                        try:
                            v_5 = self.limit - self.cursor
                            try:
                                if not self.eq_s_b(u"\u05D2"):
                                    raise lab7()
                                raise lab6()
                            except lab7: pass
                            self.cursor = self.limit - v_5
                            if not self.eq_s_b(u"\u05E9"):
                                raise lab5()
                        except lab6: pass
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_R1plus3():
                                self.cursor = self.limit - v_6
                                raise lab8()
                            if not self.slice_from(u"\u05D9\u05E1"):
                                return False
                        except lab8: pass
                        raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_4
                    if not self.__r_R1():
                        raise lab0()
                    if not self.slice_del():
                        return False

                except lab4: pass
        except lab0: pass
        self.cursor = self.limit - v_1
        v_7 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_5)
            if among_var == 0:
                raise lab9()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab9()
                if not self.slice_del():
                    return False

            else:
                if not self.__r_R1():
                    raise lab9()
                if not self.in_grouping_b(YiddishStemmer.g_consonant, 1489, 1520):
                    raise lab9()
                if not self.slice_del():
                    return False

        except lab9: pass
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_6)
            if among_var == 0:
                raise lab10()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab10()
                if not self.slice_del():
                    return False

        except lab10: pass
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        try:
            while True:
                v_10 = self.limit - self.cursor
                try:
                    try:
                        while True:
                            v_11 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                try:
                                    v_12 = self.limit - self.cursor
                                    try:
                                        if not self.eq_s_b(u"GE"):
                                            raise lab16()
                                        raise lab15()
                                    except lab16: pass
                                    self.cursor = self.limit - v_12
                                    if not self.eq_s_b(u"TSU"):
                                        raise lab14()
                                except lab15: pass
                                self.bra = self.cursor
                                if not self.slice_del():
                                    return False

                                self.cursor = self.limit - v_11
                                raise lab13()
                            except lab14: pass
                            self.cursor = self.limit - v_11
                            if self.cursor <= self.limit_backward:
                                raise lab12()
                            self.cursor -= 1
                    except lab13: pass
                    continue
                except lab12: pass
                self.cursor = self.limit - v_10
                break
        except lab11: pass
        self.cursor = self.limit - v_9
        return True

    def _stem(self):
        self.__r_prelude()
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass


class lab11(BaseException): pass


class lab12(BaseException): pass


class lab13(BaseException): pass


class lab14(BaseException): pass


class lab15(BaseException): pass


class lab16(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class FinnishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"pa", -1, 1),
        Among(u"sti", -1, 2),
        Among(u"kaan", -1, 1),
        Among(u"han", -1, 1),
        Among(u"kin", -1, 1),
        Among(u"h\u00E4n", -1, 1),
        Among(u"k\u00E4\u00E4n", -1, 1),
        Among(u"ko", -1, 1),
        Among(u"p\u00E4", -1, 1),
        Among(u"k\u00F6", -1, 1)
    ]

    a_1 = [
        Among(u"lla", -1, -1),
        Among(u"na", -1, -1),
        Among(u"ssa", -1, -1),
        Among(u"ta", -1, -1),
        Among(u"lta", 3, -1),
        Among(u"sta", 3, -1)
    ]

    a_2 = [
        Among(u"ll\u00E4", -1, -1),
        Among(u"n\u00E4", -1, -1),
        Among(u"ss\u00E4", -1, -1),
        Among(u"t\u00E4", -1, -1),
        Among(u"lt\u00E4", 3, -1),
        Among(u"st\u00E4", 3, -1)
    ]

    a_3 = [
        Among(u"lle", -1, -1),
        Among(u"ine", -1, -1)
    ]

    a_4 = [
        Among(u"nsa", -1, 3),
        Among(u"mme", -1, 3),
        Among(u"nne", -1, 3),
        Among(u"ni", -1, 2),
        Among(u"si", -1, 1),
        Among(u"an", -1, 4),
        Among(u"en", -1, 6),
        Among(u"\u00E4n", -1, 5),
        Among(u"ns\u00E4", -1, 3)
    ]

    a_5 = [
        Among(u"aa", -1, -1),
        Among(u"ee", -1, -1),
        Among(u"ii", -1, -1),
        Among(u"oo", -1, -1),
        Among(u"uu", -1, -1),
        Among(u"\u00E4\u00E4", -1, -1),
        Among(u"\u00F6\u00F6", -1, -1)
    ]

    a_6 = [
        Among(u"a", -1, 8),
        Among(u"lla", 0, -1),
        Among(u"na", 0, -1),
        Among(u"ssa", 0, -1),
        Among(u"ta", 0, -1),
        Among(u"lta", 4, -1),
        Among(u"sta", 4, -1),
        Among(u"tta", 4, 2),
        Among(u"lle", -1, -1),
        Among(u"ine", -1, -1),
        Among(u"ksi", -1, -1),
        Among(u"n", -1, 7),
        Among(u"han", 11, 1),
        Among(u"den", 11, -1, "_FinnishStemmer__r_VI"),
        Among(u"seen", 11, -1, "_FinnishStemmer__r_LONG"),
        Among(u"hen", 11, 2),
        Among(u"tten", 11, -1, "_FinnishStemmer__r_VI"),
        Among(u"hin", 11, 3),
        Among(u"siin", 11, -1, "_FinnishStemmer__r_VI"),
        Among(u"hon", 11, 4),
        Among(u"h\u00E4n", 11, 5),
        Among(u"h\u00F6n", 11, 6),
        Among(u"\u00E4", -1, 8),
        Among(u"ll\u00E4", 22, -1),
        Among(u"n\u00E4", 22, -1),
        Among(u"ss\u00E4", 22, -1),
        Among(u"t\u00E4", 22, -1),
        Among(u"lt\u00E4", 26, -1),
        Among(u"st\u00E4", 26, -1),
        Among(u"tt\u00E4", 26, 2)
    ]

    a_7 = [
        Among(u"eja", -1, -1),
        Among(u"mma", -1, 1),
        Among(u"imma", 1, -1),
        Among(u"mpa", -1, 1),
        Among(u"impa", 3, -1),
        Among(u"mmi", -1, 1),
        Among(u"immi", 5, -1),
        Among(u"mpi", -1, 1),
        Among(u"impi", 7, -1),
        Among(u"ej\u00E4", -1, -1),
        Among(u"mm\u00E4", -1, 1),
        Among(u"imm\u00E4", 10, -1),
        Among(u"mp\u00E4", -1, 1),
        Among(u"imp\u00E4", 12, -1)
    ]

    a_8 = [
        Among(u"i", -1, -1),
        Among(u"j", -1, -1)
    ]

    a_9 = [
        Among(u"mma", -1, 1),
        Among(u"imma", 0, -1)
    ]

    g_AEI = [17, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8]

    g_C = [119, 223, 119, 1]

    g_V1 = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 32]

    g_V2 = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 32]

    g_particle_end = [17, 97, 24, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 32]

    B_ending_removed = False
    S_x = ""
    I_p2 = 0
    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        if not self.go_out_grouping(FinnishStemmer.g_V1, 97, 246):
            return False
        if not self.go_in_grouping(FinnishStemmer.g_V1, 97, 246):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        if not self.go_out_grouping(FinnishStemmer.g_V1, 97, 246):
            return False
        if not self.go_in_grouping(FinnishStemmer.g_V1, 97, 246):
            return False
        self.cursor += 1
        self.I_p2 = self.cursor
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_particle_etc(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(FinnishStemmer.a_0)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            if not self.in_grouping_b(FinnishStemmer.g_particle_end, 97, 246):
                return False
        else:
            if not self.__r_R2():
                return False
        if not self.slice_del():
            return False

        return True

    def __r_possessive(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(FinnishStemmer.a_4)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            v_3 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"k"):
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_3
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_del():
                return False

            self.ket = self.cursor
            if not self.eq_s_b(u"kse"):
                return False
            self.bra = self.cursor
            if not self.slice_from(u"ksi"):
                return False
        elif among_var == 3:
            if not self.slice_del():
                return False

        elif among_var == 4:
            if self.find_among_b(FinnishStemmer.a_1) == 0:
                return False
            if not self.slice_del():
                return False

        elif among_var == 5:
            if self.find_among_b(FinnishStemmer.a_2) == 0:
                return False
            if not self.slice_del():
                return False

        else:
            if self.find_among_b(FinnishStemmer.a_3) == 0:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_LONG(self):
        if self.find_among_b(FinnishStemmer.a_5) == 0:
            return False
        return True

    def __r_VI(self):
        if not self.eq_s_b(u"i"):
            return False
        if not self.in_grouping_b(FinnishStemmer.g_V2, 97, 246):
            return False
        return True

    def __r_case_ending(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(FinnishStemmer.a_6)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            if not self.eq_s_b(u"a"):
                return False
        elif among_var == 2:
            if not self.eq_s_b(u"e"):
                return False
        elif among_var == 3:
            if not self.eq_s_b(u"i"):
                return False
        elif among_var == 4:
            if not self.eq_s_b(u"o"):
                return False
        elif among_var == 5:
            if not self.eq_s_b(u"\u00E4"):
                return False
        elif among_var == 6:
            if not self.eq_s_b(u"\u00F6"):
                return False
        elif among_var == 7:
            v_3 = self.limit - self.cursor
            try:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    try:
                        if not self.__r_LONG():
                            raise lab2()
                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_5
                    if not self.eq_s_b(u"ie"):
                        self.cursor = self.limit - v_3
                        raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_4
                if self.cursor <= self.limit_backward:
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.cursor -= 1
                self.bra = self.cursor
            except lab0: pass
        elif among_var == 8:
            if not self.in_grouping_b(FinnishStemmer.g_V1, 97, 246):
                return False
            if not self.in_grouping_b(FinnishStemmer.g_C, 98, 122):
                return False
        if not self.slice_del():
            return False

        self.B_ending_removed = True
        return True

    def __r_other_endings(self):
        if self.cursor < self.I_p2:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p2
        self.ket = self.cursor
        among_var = self.find_among_b(FinnishStemmer.a_7)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            v_3 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"po"):
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_3
        if not self.slice_del():
            return False

        return True

    def __r_i_plural(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(FinnishStemmer.a_8) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if not self.slice_del():
            return False

        return True

    def __r_t_plural(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if not self.eq_s_b(u"t"):
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        v_3 = self.limit - self.cursor
        if not self.in_grouping_b(FinnishStemmer.g_V1, 97, 246):
            self.limit_backward = v_2
            return False
        self.cursor = self.limit - v_3
        if not self.slice_del():
            return False

        self.limit_backward = v_2
        if self.cursor < self.I_p2:
            return False
        v_5 = self.limit_backward
        self.limit_backward = self.I_p2
        self.ket = self.cursor
        among_var = self.find_among_b(FinnishStemmer.a_9)
        if among_var == 0:
            self.limit_backward = v_5
            return False
        self.bra = self.cursor
        self.limit_backward = v_5
        if among_var == 1:
            v_6 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"po"):
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_6
        if not self.slice_del():
            return False

        return True

    def __r_tidy(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        v_3 = self.limit - self.cursor
        try:
            v_4 = self.limit - self.cursor
            if not self.__r_LONG():
                raise lab0()
            self.cursor = self.limit - v_4
            self.ket = self.cursor
            if self.cursor <= self.limit_backward:
                raise lab0()
            self.cursor -= 1
            self.bra = self.cursor
            if not self.slice_del():
                return False

        except lab0: pass
        self.cursor = self.limit - v_3
        v_5 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.in_grouping_b(FinnishStemmer.g_AEI, 97, 228):
                raise lab1()
            self.bra = self.cursor
            if not self.in_grouping_b(FinnishStemmer.g_C, 98, 122):
                raise lab1()
            if not self.slice_del():
                return False

        except lab1: pass
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b(u"j"):
                raise lab2()
            self.bra = self.cursor
            try:
                v_7 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"o"):
                        raise lab4()
                    raise lab3()
                except lab4: pass
                self.cursor = self.limit - v_7
                if not self.eq_s_b(u"u"):
                    raise lab2()
            except lab3: pass
            if not self.slice_del():
                return False

        except lab2: pass
        self.cursor = self.limit - v_6
        v_8 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b(u"o"):
                raise lab5()
            self.bra = self.cursor
            if not self.eq_s_b(u"j"):
                raise lab5()
            if not self.slice_del():
                return False

        except lab5: pass
        self.cursor = self.limit - v_8
        self.limit_backward = v_2
        if not self.go_in_grouping_b(FinnishStemmer.g_V1, 97, 246):
            return False
        self.ket = self.cursor
        if not self.in_grouping_b(FinnishStemmer.g_C, 98, 122):
            return False
        self.bra = self.cursor
        self.S_x = self.slice_to()
        if self.S_x == '':
            return False
        if not self.eq_s_b(self.S_x):
            return False
        if not self.slice_del():
            return False

        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_1
        self.B_ending_removed = False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_particle_etc()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_possessive()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_case_ending()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_other_endings()
        self.cursor = self.limit - v_5
        try:
            try:
                if not self.B_ending_removed:
                    raise lab1()
                v_7 = self.limit - self.cursor
                self.__r_i_plural()
                self.cursor = self.limit - v_7
                raise lab0()
            except lab1: pass
            v_8 = self.limit - self.cursor
            self.__r_t_plural()
            self.cursor = self.limit - v_8
        except lab0: pass
        v_9 = self.limit - self.cursor
        self.__r_tidy()
        self.cursor = self.limit - v_9
        self.cursor = self.limit_backward
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class FrenchStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"col", -1, -1),
        Among(u"par", -1, -1),
        Among(u"tap", -1, -1)
    ]

    a_1 = [
        Among(u"", -1, 7),
        Among(u"H", 0, 6),
        Among(u"He", 1, 4),
        Among(u"Hi", 1, 5),
        Among(u"I", 0, 1),
        Among(u"U", 0, 2),
        Among(u"Y", 0, 3)
    ]

    a_2 = [
        Among(u"iqU", -1, 3),
        Among(u"abl", -1, 3),
        Among(u"I\u00E8r", -1, 4),
        Among(u"i\u00E8r", -1, 4),
        Among(u"eus", -1, 2),
        Among(u"iv", -1, 1)
    ]

    a_3 = [
        Among(u"ic", -1, 2),
        Among(u"abil", -1, 1),
        Among(u"iv", -1, 3)
    ]

    a_4 = [
        Among(u"iqUe", -1, 1),
        Among(u"atrice", -1, 2),
        Among(u"ance", -1, 1),
        Among(u"ence", -1, 5),
        Among(u"logie", -1, 3),
        Among(u"able", -1, 1),
        Among(u"isme", -1, 1),
        Among(u"euse", -1, 11),
        Among(u"iste", -1, 1),
        Among(u"ive", -1, 8),
        Among(u"if", -1, 8),
        Among(u"usion", -1, 4),
        Among(u"ation", -1, 2),
        Among(u"ution", -1, 4),
        Among(u"ateur", -1, 2),
        Among(u"iqUes", -1, 1),
        Among(u"atrices", -1, 2),
        Among(u"ances", -1, 1),
        Among(u"ences", -1, 5),
        Among(u"logies", -1, 3),
        Among(u"ables", -1, 1),
        Among(u"ismes", -1, 1),
        Among(u"euses", -1, 11),
        Among(u"istes", -1, 1),
        Among(u"ives", -1, 8),
        Among(u"ifs", -1, 8),
        Among(u"usions", -1, 4),
        Among(u"ations", -1, 2),
        Among(u"utions", -1, 4),
        Among(u"ateurs", -1, 2),
        Among(u"ments", -1, 15),
        Among(u"ements", 30, 6),
        Among(u"issements", 31, 12),
        Among(u"it\u00E9s", -1, 7),
        Among(u"ment", -1, 15),
        Among(u"ement", 34, 6),
        Among(u"issement", 35, 12),
        Among(u"amment", 34, 13),
        Among(u"emment", 34, 14),
        Among(u"aux", -1, 10),
        Among(u"eaux", 39, 9),
        Among(u"eux", -1, 1),
        Among(u"it\u00E9", -1, 7)
    ]

    a_5 = [
        Among(u"ira", -1, 1),
        Among(u"ie", -1, 1),
        Among(u"isse", -1, 1),
        Among(u"issante", -1, 1),
        Among(u"i", -1, 1),
        Among(u"irai", 4, 1),
        Among(u"ir", -1, 1),
        Among(u"iras", -1, 1),
        Among(u"ies", -1, 1),
        Among(u"\u00EEmes", -1, 1),
        Among(u"isses", -1, 1),
        Among(u"issantes", -1, 1),
        Among(u"\u00EEtes", -1, 1),
        Among(u"is", -1, 1),
        Among(u"irais", 13, 1),
        Among(u"issais", 13, 1),
        Among(u"irions", -1, 1),
        Among(u"issions", -1, 1),
        Among(u"irons", -1, 1),
        Among(u"issons", -1, 1),
        Among(u"issants", -1, 1),
        Among(u"it", -1, 1),
        Among(u"irait", 21, 1),
        Among(u"issait", 21, 1),
        Among(u"issant", -1, 1),
        Among(u"iraIent", -1, 1),
        Among(u"issaIent", -1, 1),
        Among(u"irent", -1, 1),
        Among(u"issent", -1, 1),
        Among(u"iront", -1, 1),
        Among(u"\u00EEt", -1, 1),
        Among(u"iriez", -1, 1),
        Among(u"issiez", -1, 1),
        Among(u"irez", -1, 1),
        Among(u"issez", -1, 1)
    ]

    a_6 = [
        Among(u"a", -1, 3),
        Among(u"era", 0, 2),
        Among(u"asse", -1, 3),
        Among(u"ante", -1, 3),
        Among(u"\u00E9e", -1, 2),
        Among(u"ai", -1, 3),
        Among(u"erai", 5, 2),
        Among(u"er", -1, 2),
        Among(u"as", -1, 3),
        Among(u"eras", 8, 2),
        Among(u"\u00E2mes", -1, 3),
        Among(u"asses", -1, 3),
        Among(u"antes", -1, 3),
        Among(u"\u00E2tes", -1, 3),
        Among(u"\u00E9es", -1, 2),
        Among(u"ais", -1, 3),
        Among(u"erais", 15, 2),
        Among(u"ions", -1, 1),
        Among(u"erions", 17, 2),
        Among(u"assions", 17, 3),
        Among(u"erons", -1, 2),
        Among(u"ants", -1, 3),
        Among(u"\u00E9s", -1, 2),
        Among(u"ait", -1, 3),
        Among(u"erait", 23, 2),
        Among(u"ant", -1, 3),
        Among(u"aIent", -1, 3),
        Among(u"eraIent", 26, 2),
        Among(u"\u00E8rent", -1, 2),
        Among(u"assent", -1, 3),
        Among(u"eront", -1, 2),
        Among(u"\u00E2t", -1, 3),
        Among(u"ez", -1, 2),
        Among(u"iez", 32, 2),
        Among(u"eriez", 33, 2),
        Among(u"assiez", 33, 3),
        Among(u"erez", 32, 2),
        Among(u"\u00E9", -1, 2)
    ]

    a_7 = [
        Among(u"e", -1, 3),
        Among(u"I\u00E8re", 0, 2),
        Among(u"i\u00E8re", 0, 2),
        Among(u"ion", -1, 1),
        Among(u"Ier", -1, 2),
        Among(u"ier", -1, 2)
    ]

    a_8 = [
        Among(u"ell", -1, -1),
        Among(u"eill", -1, -1),
        Among(u"enn", -1, -1),
        Among(u"onn", -1, -1),
        Among(u"ett", -1, -1)
    ]

    g_v = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128, 130, 103, 8, 5]

    g_keep_with_s = [1, 65, 20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128]

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                try:
                    while True:
                        v_2 = self.cursor
                        try:
                            try:
                                v_3 = self.cursor
                                try:
                                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                        raise lab4()
                                    self.bra = self.cursor
                                    try:
                                        v_4 = self.cursor
                                        try:
                                            if not self.eq_s(u"u"):
                                                raise lab6()
                                            self.ket = self.cursor
                                            if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                raise lab6()
                                            if not self.slice_from(u"U"):
                                                return False
                                            raise lab5()
                                        except lab6: pass
                                        self.cursor = v_4
                                        try:
                                            if not self.eq_s(u"i"):
                                                raise lab7()
                                            self.ket = self.cursor
                                            if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                raise lab7()
                                            if not self.slice_from(u"I"):
                                                return False
                                            raise lab5()
                                        except lab7: pass
                                        self.cursor = v_4
                                        if not self.eq_s(u"y"):
                                            raise lab4()
                                        self.ket = self.cursor
                                        if not self.slice_from(u"Y"):
                                            return False
                                    except lab5: pass
                                    raise lab3()
                                except lab4: pass
                                self.cursor = v_3
                                try:
                                    self.bra = self.cursor
                                    if not self.eq_s(u"\u00EB"):
                                        raise lab8()
                                    self.ket = self.cursor
                                    if not self.slice_from(u"He"):
                                        return False
                                    raise lab3()
                                except lab8: pass
                                self.cursor = v_3
                                try:
                                    self.bra = self.cursor
                                    if not self.eq_s(u"\u00EF"):
                                        raise lab9()
                                    self.ket = self.cursor
                                    if not self.slice_from(u"Hi"):
                                        return False
                                    raise lab3()
                                except lab9: pass
                                self.cursor = v_3
                                try:
                                    self.bra = self.cursor
                                    if not self.eq_s(u"y"):
                                        raise lab10()
                                    self.ket = self.cursor
                                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                        raise lab10()
                                    if not self.slice_from(u"Y"):
                                        return False
                                    raise lab3()
                                except lab10: pass
                                self.cursor = v_3
                                if not self.eq_s(u"q"):
                                    raise lab2()
                                self.bra = self.cursor
                                if not self.eq_s(u"u"):
                                    raise lab2()
                                self.ket = self.cursor
                                if not self.slice_from(u"U"):
                                    return False
                            except lab3: pass
                            self.cursor = v_2
                            raise lab1()
                        except lab2: pass
                        self.cursor = v_2
                        if self.cursor >= self.limit:
                            raise lab0()
                        self.cursor += 1
                except lab1: pass
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            try:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                        raise lab2()
                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                        raise lab2()
                    if self.cursor >= self.limit:
                        raise lab2()
                    self.cursor += 1
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                try:
                    if self.find_among(FrenchStemmer.a_0) == 0:
                        raise lab3()
                    raise lab1()
                except lab3: pass
                self.cursor = v_2
                if self.cursor >= self.limit:
                    raise lab0()
                self.cursor += 1
                if not self.go_out_grouping(FrenchStemmer.g_v, 97, 251):
                    raise lab0()
                self.cursor += 1
            except lab1: pass
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_3 = self.cursor
        try:
            if not self.go_out_grouping(FrenchStemmer.g_v, 97, 251):
                raise lab4()
            self.cursor += 1
            if not self.go_in_grouping(FrenchStemmer.g_v, 97, 251):
                raise lab4()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(FrenchStemmer.g_v, 97, 251):
                raise lab4()
            self.cursor += 1
            if not self.go_in_grouping(FrenchStemmer.g_v, 97, 251):
                raise lab4()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab4: pass
        self.cursor = v_3
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(FrenchStemmer.a_1)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"u"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"y"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"\u00EB"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"\u00EF"):
                        return False
                elif among_var == 6:
                    if not self.slice_del():
                        return False

                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        if not self.I_pV <= self.cursor:
            return False
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_R2():
                            raise lab2()
                        if not self.slice_del():
                            return False

                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_2
                    if not self.slice_from(u"iqU"):
                        return False
                except lab1: pass
            except lab0: pass
        elif among_var == 3:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"log"):
                return False
        elif among_var == 4:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"u"):
                return False
        elif among_var == 5:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"ent"):
                return False
        elif among_var == 6:
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(FrenchStemmer.a_2)
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab3()
                self.bra = self.cursor
                if among_var == 1:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    if not self.slice_del():
                        return False

                    self.ket = self.cursor
                    if not self.eq_s_b(u"at"):
                        self.cursor = self.limit - v_3
                        raise lab3()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    if not self.slice_del():
                        return False

                elif among_var == 2:
                    try:
                        v_4 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab5()
                            if not self.slice_del():
                                return False

                            raise lab4()
                        except lab5: pass
                        self.cursor = self.limit - v_4
                        if not self.__r_R1():
                            self.cursor = self.limit - v_3
                            raise lab3()
                        if not self.slice_from(u"eux"):
                            return False
                    except lab4: pass
                elif among_var == 3:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    if not self.slice_del():
                        return False

                else:
                    if not self.__r_RV():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    if not self.slice_from(u"i"):
                        return False
            except lab3: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_5 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(FrenchStemmer.a_3)
                if among_var == 0:
                    self.cursor = self.limit - v_5
                    raise lab6()
                self.bra = self.cursor
                if among_var == 1:
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab8()
                            if not self.slice_del():
                                return False

                            raise lab7()
                        except lab8: pass
                        self.cursor = self.limit - v_6
                        if not self.slice_from(u"abl"):
                            return False
                    except lab7: pass
                elif among_var == 2:
                    try:
                        v_7 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab10()
                            if not self.slice_del():
                                return False

                            raise lab9()
                        except lab10: pass
                        self.cursor = self.limit - v_7
                        if not self.slice_from(u"iqU"):
                            return False
                    except lab9: pass
                else:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_5
                        raise lab6()
                    if not self.slice_del():
                        return False

            except lab6: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_8 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"at"):
                    self.cursor = self.limit - v_8
                    raise lab11()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_8
                    raise lab11()
                if not self.slice_del():
                    return False

                self.ket = self.cursor
                if not self.eq_s_b(u"ic"):
                    self.cursor = self.limit - v_8
                    raise lab11()
                self.bra = self.cursor
                try:
                    v_9 = self.limit - self.cursor
                    try:
                        if not self.__r_R2():
                            raise lab13()
                        if not self.slice_del():
                            return False

                        raise lab12()
                    except lab13: pass
                    self.cursor = self.limit - v_9
                    if not self.slice_from(u"iqU"):
                        return False
                except lab12: pass
            except lab11: pass
        elif among_var == 9:
            if not self.slice_from(u"eau"):
                return False
        elif among_var == 10:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"al"):
                return False
        elif among_var == 11:
            try:
                v_10 = self.limit - self.cursor
                try:
                    if not self.__r_R2():
                        raise lab15()
                    if not self.slice_del():
                        return False

                    raise lab14()
                except lab15: pass
                self.cursor = self.limit - v_10
                if not self.__r_R1():
                    return False
                if not self.slice_from(u"eux"):
                    return False
            except lab14: pass
        elif among_var == 12:
            if not self.__r_R1():
                return False
            if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
                return False
            if not self.slice_del():
                return False

        elif among_var == 13:
            if not self.__r_RV():
                return False
            if not self.slice_from(u"ant"):
                return False
            return False
        elif among_var == 14:
            if not self.__r_RV():
                return False
            if not self.slice_from(u"ent"):
                return False
            return False
        else:
            v_11 = self.limit - self.cursor
            if not self.in_grouping_b(FrenchStemmer.g_v, 97, 251):
                return False
            if not self.__r_RV():
                return False
            self.cursor = self.limit - v_11
            if not self.slice_del():
                return False

            return False
        return True

    def __r_i_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(FrenchStemmer.a_5) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        v_3 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"H"):
                raise lab0()
            self.limit_backward = v_2
            return False
        except lab0: pass
        self.cursor = self.limit - v_3
        if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
            self.limit_backward = v_2
            return False
        if not self.slice_del():
            return False

        self.limit_backward = v_2
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_6)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                self.limit_backward = v_2
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"e"):
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
                if not self.slice_del():
                    return False

            except lab0: pass
        self.limit_backward = v_2
        return True

    def __r_residual_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b(u"s"):
                self.cursor = self.limit - v_1
                raise lab0()
            self.bra = self.cursor
            v_2 = self.limit - self.cursor
            try:
                v_3 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"Hi"):
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_3
                if not self.out_grouping_b(FrenchStemmer.g_keep_with_s, 97, 232):
                    self.cursor = self.limit - v_1
                    raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_2
            if not self.slice_del():
                return False

        except lab0: pass
        if self.cursor < self.I_pV:
            return False
        v_5 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_7)
        if among_var == 0:
            self.limit_backward = v_5
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                self.limit_backward = v_5
                return False
            try:
                v_6 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"s"):
                        raise lab4()
                    raise lab3()
                except lab4: pass
                self.cursor = self.limit - v_6
                if not self.eq_s_b(u"t"):
                    self.limit_backward = v_5
                    return False
            except lab3: pass
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"i"):
                return False
        else:
            if not self.slice_del():
                return False

        self.limit_backward = v_5
        return True

    def __r_un_double(self):
        v_1 = self.limit - self.cursor
        if self.find_among_b(FrenchStemmer.a_8) == 0:
            return False
        self.cursor = self.limit - v_1
        self.ket = self.cursor
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_un_accent(self):
        v_1 = 1
        while True:
            try:
                if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
                    raise lab0()
                v_1 -= 1
                continue
            except lab0: pass
            break
        if v_1 > 0:
            return False
        self.ket = self.cursor
        try:
            v_3 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"\u00E9"):
                    raise lab2()
                raise lab1()
            except lab2: pass
            self.cursor = self.limit - v_3
            if not self.eq_s_b(u"\u00E8"):
                return False
        except lab1: pass
        self.bra = self.cursor
        if not self.slice_from(u"e"):
            return False
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_standard_suffix():
                                raise lab4()
                            raise lab3()
                        except lab4: pass
                        self.cursor = self.limit - v_6
                        try:
                            if not self.__r_i_verb_suffix():
                                raise lab5()
                            raise lab3()
                        except lab5: pass
                        self.cursor = self.limit - v_6
                        if not self.__r_verb_suffix():
                            raise lab2()
                    except lab3: pass
                    self.cursor = self.limit - v_5
                    v_7 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        try:
                            v_8 = self.limit - self.cursor
                            try:
                                if not self.eq_s_b(u"Y"):
                                    raise lab8()
                                self.bra = self.cursor
                                if not self.slice_from(u"i"):
                                    return False
                                raise lab7()
                            except lab8: pass
                            self.cursor = self.limit - v_8
                            if not self.eq_s_b(u"\u00E7"):
                                self.cursor = self.limit - v_7
                                raise lab6()
                            self.bra = self.cursor
                            if not self.slice_from(u"c"):
                                return False
                        except lab7: pass
                    except lab6: pass
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                if not self.__r_residual_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        v_9 = self.limit - self.cursor
        self.__r_un_double()
        self.cursor = self.limit - v_9
        v_10 = self.limit - self.cursor
        self.__r_un_accent()
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        v_11 = self.cursor
        self.__r_postlude()
        self.cursor = v_11
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass


class lab11(BaseException): pass


class lab12(BaseException): pass


class lab13(BaseException): pass


class lab14(BaseException): pass


class lab15(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class HungarianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"cs", -1, -1),
        Among(u"dzs", -1, -1),
        Among(u"gy", -1, -1),
        Among(u"ly", -1, -1),
        Among(u"ny", -1, -1),
        Among(u"sz", -1, -1),
        Among(u"ty", -1, -1),
        Among(u"zs", -1, -1)
    ]

    a_1 = [
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 2)
    ]

    a_2 = [
        Among(u"bb", -1, -1),
        Among(u"cc", -1, -1),
        Among(u"dd", -1, -1),
        Among(u"ff", -1, -1),
        Among(u"gg", -1, -1),
        Among(u"jj", -1, -1),
        Among(u"kk", -1, -1),
        Among(u"ll", -1, -1),
        Among(u"mm", -1, -1),
        Among(u"nn", -1, -1),
        Among(u"pp", -1, -1),
        Among(u"rr", -1, -1),
        Among(u"ccs", -1, -1),
        Among(u"ss", -1, -1),
        Among(u"zzs", -1, -1),
        Among(u"tt", -1, -1),
        Among(u"vv", -1, -1),
        Among(u"ggy", -1, -1),
        Among(u"lly", -1, -1),
        Among(u"nny", -1, -1),
        Among(u"tty", -1, -1),
        Among(u"ssz", -1, -1),
        Among(u"zz", -1, -1)
    ]

    a_3 = [
        Among(u"al", -1, 1),
        Among(u"el", -1, 1)
    ]

    a_4 = [
        Among(u"ba", -1, -1),
        Among(u"ra", -1, -1),
        Among(u"be", -1, -1),
        Among(u"re", -1, -1),
        Among(u"ig", -1, -1),
        Among(u"nak", -1, -1),
        Among(u"nek", -1, -1),
        Among(u"val", -1, -1),
        Among(u"vel", -1, -1),
        Among(u"ul", -1, -1),
        Among(u"n\u00E1l", -1, -1),
        Among(u"n\u00E9l", -1, -1),
        Among(u"b\u00F3l", -1, -1),
        Among(u"r\u00F3l", -1, -1),
        Among(u"t\u00F3l", -1, -1),
        Among(u"\u00FCl", -1, -1),
        Among(u"b\u0151l", -1, -1),
        Among(u"r\u0151l", -1, -1),
        Among(u"t\u0151l", -1, -1),
        Among(u"n", -1, -1),
        Among(u"an", 19, -1),
        Among(u"ban", 20, -1),
        Among(u"en", 19, -1),
        Among(u"ben", 22, -1),
        Among(u"k\u00E9ppen", 22, -1),
        Among(u"on", 19, -1),
        Among(u"\u00F6n", 19, -1),
        Among(u"k\u00E9pp", -1, -1),
        Among(u"kor", -1, -1),
        Among(u"t", -1, -1),
        Among(u"at", 29, -1),
        Among(u"et", 29, -1),
        Among(u"k\u00E9nt", 29, -1),
        Among(u"ank\u00E9nt", 32, -1),
        Among(u"enk\u00E9nt", 32, -1),
        Among(u"onk\u00E9nt", 32, -1),
        Among(u"ot", 29, -1),
        Among(u"\u00E9rt", 29, -1),
        Among(u"\u00F6t", 29, -1),
        Among(u"hez", -1, -1),
        Among(u"hoz", -1, -1),
        Among(u"h\u00F6z", -1, -1),
        Among(u"v\u00E1", -1, -1),
        Among(u"v\u00E9", -1, -1)
    ]

    a_5 = [
        Among(u"\u00E1n", -1, 2),
        Among(u"\u00E9n", -1, 1),
        Among(u"\u00E1nk\u00E9nt", -1, 2)
    ]

    a_6 = [
        Among(u"stul", -1, 1),
        Among(u"astul", 0, 1),
        Among(u"\u00E1stul", 0, 2),
        Among(u"st\u00FCl", -1, 1),
        Among(u"est\u00FCl", 3, 1),
        Among(u"\u00E9st\u00FCl", 3, 3)
    ]

    a_7 = [
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 1)
    ]

    a_8 = [
        Among(u"k", -1, 3),
        Among(u"ak", 0, 3),
        Among(u"ek", 0, 3),
        Among(u"ok", 0, 3),
        Among(u"\u00E1k", 0, 1),
        Among(u"\u00E9k", 0, 2),
        Among(u"\u00F6k", 0, 3)
    ]

    a_9 = [
        Among(u"\u00E9i", -1, 1),
        Among(u"\u00E1\u00E9i", 0, 3),
        Among(u"\u00E9\u00E9i", 0, 2),
        Among(u"\u00E9", -1, 1),
        Among(u"k\u00E9", 3, 1),
        Among(u"ak\u00E9", 4, 1),
        Among(u"ek\u00E9", 4, 1),
        Among(u"ok\u00E9", 4, 1),
        Among(u"\u00E1k\u00E9", 4, 3),
        Among(u"\u00E9k\u00E9", 4, 2),
        Among(u"\u00F6k\u00E9", 4, 1),
        Among(u"\u00E9\u00E9", 3, 2)
    ]

    a_10 = [
        Among(u"a", -1, 1),
        Among(u"ja", 0, 1),
        Among(u"d", -1, 1),
        Among(u"ad", 2, 1),
        Among(u"ed", 2, 1),
        Among(u"od", 2, 1),
        Among(u"\u00E1d", 2, 2),
        Among(u"\u00E9d", 2, 3),
        Among(u"\u00F6d", 2, 1),
        Among(u"e", -1, 1),
        Among(u"je", 9, 1),
        Among(u"nk", -1, 1),
        Among(u"unk", 11, 1),
        Among(u"\u00E1nk", 11, 2),
        Among(u"\u00E9nk", 11, 3),
        Among(u"\u00FCnk", 11, 1),
        Among(u"uk", -1, 1),
        Among(u"juk", 16, 1),
        Among(u"\u00E1juk", 17, 2),
        Among(u"\u00FCk", -1, 1),
        Among(u"j\u00FCk", 19, 1),
        Among(u"\u00E9j\u00FCk", 20, 3),
        Among(u"m", -1, 1),
        Among(u"am", 22, 1),
        Among(u"em", 22, 1),
        Among(u"om", 22, 1),
        Among(u"\u00E1m", 22, 2),
        Among(u"\u00E9m", 22, 3),
        Among(u"o", -1, 1),
        Among(u"\u00E1", -1, 2),
        Among(u"\u00E9", -1, 3)
    ]

    a_11 = [
        Among(u"id", -1, 1),
        Among(u"aid", 0, 1),
        Among(u"jaid", 1, 1),
        Among(u"eid", 0, 1),
        Among(u"jeid", 3, 1),
        Among(u"\u00E1id", 0, 2),
        Among(u"\u00E9id", 0, 3),
        Among(u"i", -1, 1),
        Among(u"ai", 7, 1),
        Among(u"jai", 8, 1),
        Among(u"ei", 7, 1),
        Among(u"jei", 10, 1),
        Among(u"\u00E1i", 7, 2),
        Among(u"\u00E9i", 7, 3),
        Among(u"itek", -1, 1),
        Among(u"eitek", 14, 1),
        Among(u"jeitek", 15, 1),
        Among(u"\u00E9itek", 14, 3),
        Among(u"ik", -1, 1),
        Among(u"aik", 18, 1),
        Among(u"jaik", 19, 1),
        Among(u"eik", 18, 1),
        Among(u"jeik", 21, 1),
        Among(u"\u00E1ik", 18, 2),
        Among(u"\u00E9ik", 18, 3),
        Among(u"ink", -1, 1),
        Among(u"aink", 25, 1),
        Among(u"jaink", 26, 1),
        Among(u"eink", 25, 1),
        Among(u"jeink", 28, 1),
        Among(u"\u00E1ink", 25, 2),
        Among(u"\u00E9ink", 25, 3),
        Among(u"aitok", -1, 1),
        Among(u"jaitok", 32, 1),
        Among(u"\u00E1itok", -1, 2),
        Among(u"im", -1, 1),
        Among(u"aim", 35, 1),
        Among(u"jaim", 36, 1),
        Among(u"eim", 35, 1),
        Among(u"jeim", 38, 1),
        Among(u"\u00E1im", 35, 2),
        Among(u"\u00E9im", 35, 3)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 17, 36, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1]

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        try:
            v_1 = self.cursor
            try:
                if not self.in_grouping(HungarianStemmer.g_v, 97, 369):
                    raise lab1()
                if not self.go_in_grouping(HungarianStemmer.g_v, 97, 369):
                    raise lab1()
                try:
                    v_2 = self.cursor
                    try:
                        if self.find_among(HungarianStemmer.a_0) == 0:
                            raise lab3()
                        raise lab2()
                    except lab3: pass
                    self.cursor = v_2
                    if self.cursor >= self.limit:
                        raise lab1()
                    self.cursor += 1
                except lab2: pass
                self.I_p1 = self.cursor
                raise lab0()
            except lab1: pass
            self.cursor = v_1
            if not self.out_grouping(HungarianStemmer.g_v, 97, 369):
                return False
            if not self.go_out_grouping(HungarianStemmer.g_v, 97, 369):
                return False
            self.cursor += 1
            self.I_p1 = self.cursor
        except lab0: pass
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_v_ending(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_1)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"a"):
                return False
        else:
            if not self.slice_from(u"e"):
                return False
        return True

    def __r_double(self):
        v_1 = self.limit - self.cursor
        if self.find_among_b(HungarianStemmer.a_2) == 0:
            return False
        self.cursor = self.limit - v_1
        return True

    def __r_undouble(self):
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.ket = self.cursor
        c = self.cursor - 1
        if c < self.limit_backward:
            return False
        self.cursor = c
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_instrum(self):
        self.ket = self.cursor
        if self.find_among_b(HungarianStemmer.a_3) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.__r_double():
            return False
        if not self.slice_del():
            return False

        if not self.__r_undouble():
            return False
        return True

    def __r_case(self):
        self.ket = self.cursor
        if self.find_among_b(HungarianStemmer.a_4) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.slice_del():
            return False

        if not self.__r_v_ending():
            return False
        return True

    def __r_case_special(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_5)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"e"):
                return False
        else:
            if not self.slice_from(u"a"):
                return False
        return True

    def __r_case_other(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_6)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"a"):
                return False
        else:
            if not self.slice_from(u"e"):
                return False
        return True

    def __r_factive(self):
        self.ket = self.cursor
        if self.find_among_b(HungarianStemmer.a_7) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.__r_double():
            return False
        if not self.slice_del():
            return False

        if not self.__r_undouble():
            return False
        return True

    def __r_plural(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"a"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"e"):
                return False
        else:
            if not self.slice_del():
                return False

        return True

    def __r_owned(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_9)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"e"):
                return False
        else:
            if not self.slice_from(u"a"):
                return False
        return True

    def __r_sing_owner(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_10)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"a"):
                return False
        else:
            if not self.slice_from(u"e"):
                return False
        return True

    def __r_plur_owner(self):
        self.ket = self.cursor
        among_var = self.find_among_b(HungarianStemmer.a_11)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"a"):
                return False
        else:
            if not self.slice_from(u"e"):
                return False
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_1
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_instrum()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_case()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_case_special()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_case_other()
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        self.__r_factive()
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        self.__r_owned()
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        self.__r_sing_owner()
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        self.__r_plur_owner()
        self.cursor = self.limit - v_9
        v_10 = self.limit - self.cursor
        self.__r_plural()
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass
